# Copyright (C) 2006 Ulrik Sverdrup
#
#   dragbox -- Commandline drag-and-drop tool for GNOME
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
# USA 

import pygtk
pygtk.require('2.0') 
import gtk
from os import path

FILE_TYPE = "file"
TEXT_TYPE = "text"
DATA_TYPE = "data"

TARGET_TYPE_TEXT = 80
TARGET_TYPE_URI_LIST = 81

text_targets = gtk.target_list_add_text_targets([], TARGET_TYPE_TEXT)
file_targets = gtk.target_list_add_uri_targets([], TARGET_TYPE_URI_LIST)


class dragitem(object):
	"""
	dragitem object, represents one draggable item
	
	"""
	
	def __init__(self, obj, type=TEXT_TYPE): 
		self.widget = gtk.Button("Placeholder Button", use_underline=False)
		self.widget.set_relief(gtk.RELIEF_NONE)
		label = self.widget.get_children()[0] #this is a label
		label.set_justify(gtk.JUSTIFY_LEFT)

		self.set_object(obj, type)
	def get_widget(self): 
		""" 
		accessor
		returns the represented widget 
		 
		""" 
		return self.widget
	def get_type(self):
		""" 
		Returns the data type of the represented object 
		""" 
		return self.type
	def get_targets(self):
		""" 
		Returns the appropriate drag types for the represented 
		object 
		"""  
		if self.type is TEXT_TYPE:
			return text_targets
		elif self.type is FILE_TYPE: 
			return file_targets	+ text_targets  

	def get_object(self): 
		return self.object
	def set_object(self, obj, type=TEXT_TYPE):
		""" 
		Setter, sets the represented object
		on the dragitem 
		 
		""" 
		self.object = obj;
		self.type = type
		name = obj;
		
		# Func to wrap the some text
		def wrap_text(text, maxlines):
			from textwrap import wrap
			maxlen = 35;
			name = text.lstrip().rstrip() # Strip lead/tail whitesp
			
			wrapped = wrap(name, maxlen)
			lines = wrapped[:maxlines]
			sep = "\n"
			name = sep.join(lines)
			if len(wrapped) > maxlines:
				name += "..."
			return name
		
		if type is FILE_TYPE:
			show_parent = False
			(pa, item) = path.split(self.object)
			if show_parent:
				(pa, item2) = path.split(pa)
				if not item2: item2 = "/" # filesys root
				if pa is "/": item2 = pa + item2
				name = path.join(item2, item)
				name = wrap_text(name, 10)
			else:
				if not item: item = pa;
				name = item
			#workaround underline bug
			name = name.replace("_", "__")
		else:
			name = wrap_text(name, 3)
			name = "\"" + name + "\""
		  
		self.widget.set_label(name)
		#does not appear to work
		#self.widget.set_use_underline(False)
		
		maxtiplen = 512
		tip = obj
		if len(tip) > maxtiplen:
			tip = tip[:maxtiplen] + "\n..."
		tools = gtk.Tooltips()
		tools.set_tip(self.widget, tip)

		if type is FILE_TYPE:
			
			from utils import get_icon_for_uri
			uri =self.get_uri()
			
			icon = get_icon_for_uri(uri, icon_size=48)
			icon_widget = gtk.Image()
			icon_widget.set_from_pixbuf(icon)
			self.widget.set_image(icon_widget)
		self.widget.set_alignment(0, 0.5)
	
	def write_to_selection(self, selection, target_type):
		""" 
		Tells the dragitem to write its representation to 
		the selection with the type target_type 
		""" 
		if target_type == TARGET_TYPE_URI_LIST and self.type == FILE_TYPE: 
			uri = self.get_uri()
			out = [uri]
			
			selection.set_uris(out)
		elif target_type == TARGET_TYPE_TEXT:
			selection.set_text(self.object, -1)
		else:
			from utils import print_error 
			print_error("error with drag types?")
			
	def print_to_stdout(self, print_path=False, null_terminate=False):
		""" 
		Print the represented object to stdout 
		"""
		from sys import stdout 
		if self.type is TEXT_TYPE:
			text = self.object
			stdout.write(text)
			
		elif self.type is FILE_TYPE:
			if print_path:
				text = self.get_object()
			else:
				text = self.get_uri()
			stdout.write(text)
		if null_terminate:
			stdout.write("\0")
		else:
			stdout.write("\n")  # print empty line at end of snippet
		
	def get_uri(self): 
		""" 
		Returns the uri of the dragitem
		Throws if not FILE_TYPE 
		""" 
		assert self.type is FILE_TYPE, "check_still_exists called on non-file"
		from gnomevfs import get_uri_from_local_path
		uri = get_uri_from_local_path(self.object)
		return uri 
	def get_human_readable(self): 
		""" 
		Returns the preferred readable form of the dragitem
		""" 
		return str(self.object)
	def still_exists(self):
		""" 
		Checks if the represented file still exists
		returns 0 if it does not exist 
		""" 
		uri = self.get_uri()
		from gnomevfs import exists
		return exists(uri) 
		 
